#pragma once

#include <platform.h>

class CTraceBuffer {
public:
  CTraceBuffer(unsigned int sensorIndex) {
    m_pDataPoints = NULL;
    m_length = 0;
    m_head = 0;
    m_tail = 0;
    m_maxLength = 0;
    m_bMinMaxMeanValid = FALSE;
    m_sensorIndex = sensorIndex;
  }
  virtual ~CTraceBuffer() {
    if (NULL != m_pDataPoints) {
      delete[] m_pDataPoints;
      m_pDataPoints = NULL;
    }
  }

  void setSize(
    unsigned int size)
  {
    float* pNewPoints;
    unsigned int from;

    ASSERT(size > 0);
    pNewPoints = new float[size];
    if (size <= m_length) {
      ASSERT(size <= m_maxLength);
      from = moduloSub(m_tail, size, m_maxLength);
      m_sum = copyData(pNewPoints, from, size);
      m_length = size;
      m_head = 0;
      m_tail = 0;
    } else {
      ASSERT(m_length <= m_maxLength);
      from = moduloSub(m_tail, m_length, m_maxLength);
      m_sum = copyData(pNewPoints, from, m_length);
      m_head = 0;
      m_tail = m_length;
    }
    m_maxLength = size;
    delete[] m_pDataPoints;
    m_pDataPoints = pNewPoints;
  }

  void addPoint(float point) {
    ASSERT(NULL != m_pDataPoints);
    ASSERT(m_maxLength > 0);
    m_pDataPoints[m_tail] = point;
    m_sum += point;
    if (!m_bMinMaxMeanValid) {
      m_min = m_max = point;
      m_bMinMaxMeanValid = TRUE;
    } else {
      if (point < m_min) {
        m_min = point;
      }
      if (point > m_max) {
        m_max = point;
      }
    }

    m_tail = moduloAdd(m_tail, 1, m_maxLength);
    if (m_length == m_maxLength) {
      m_sum -= m_pDataPoints[m_head];
      m_head = moduloAdd(m_head, 1, m_maxLength);
    } else {
      m_length++;
    }
  }

  unsigned int getLength() const {
    return m_length;
  }

  float getPoint(unsigned int position) const {
    ASSERT(NULL != m_pDataPoints);
    ASSERT(position < m_length);

    position = moduloAdd(position, m_head, m_maxLength);
    return m_pDataPoints[position];
  }

  unsigned int getSensorIndex() const {
    return m_sensorIndex;
  }

  float getMin() const {
    ASSERT(m_bMinMaxMeanValid);
    return m_min;
  }

  float getMax() const {
    ASSERT(m_bMinMaxMeanValid);
    return m_max;
  }

  float getMean() const {
    ASSERT(m_bMinMaxMeanValid);
    return m_sum / (float)m_length;
  }

  void clear() {
    m_head = 0;
    m_tail = 0;
    m_length = 0;
    m_sum = 0.0f;
    m_bMinMaxMeanValid = FALSE;
  }

protected:
  unsigned int moduloSub(
    unsigned int a,
    unsigned int b,
    unsigned int m) const
  {
    return (a >= b) ? a - b : m + a - b;
  }

  unsigned int moduloAdd(
    unsigned int a,
    unsigned int b,
    unsigned int m) const
  {
    unsigned int sum = a + b;

    return (sum < m) ? sum : sum - m;
  }

  float copyData(
    float* pBuffer,
    unsigned int from,
    unsigned int n)
  {
    float* p = m_pDataPoints + from;
    float sum = 0.0f;
    float v;

    ASSERT(n == 0 || NULL != m_pDataPoints);
    ASSERT(n <= m_length);

    while (n) {
      v = *p++;
      sum += v;
      *pBuffer++ = v;
      from++;
      if (from >= m_maxLength) {
        from -= m_maxLength;
        p = m_pDataPoints;
      }
      n--;
    }

    return sum;
  }

protected:
  unsigned int m_maxLength;
  unsigned int m_length;
  unsigned int m_head, m_tail;
  bool_t m_bMinMaxMeanValid;
  float* m_pDataPoints;
  float m_min, m_max;
  float m_sum;
  unsigned int m_sensorIndex;
};
