#include "scopekey.h"

static const TCHAR* g_pMessage = "Key to traces:";

CScopeKey::CScopeKey(
  IGraphWndToolbar* pParent)
{
  m_pParent = pParent;

  set_decorated(false);

  m_drawing.signal_expose_event().connect(sigc::mem_fun(*this, &CScopeKey::onExpose), false);
  signal_realize().connect(sigc::mem_fun(*this, &CScopeKey::onRealize), false);

  add(m_drawing);
  m_drawing.show();
}

CScopeKey::~CScopeKey()
{
}

bool
CScopeKey::onExpose(GdkEventExpose* pEvent) {
  ISysMonWnd* pMainWnd = m_pParent->getParent()->getParent();
  IScopeWnd* pScopeWnd = m_pParent->getParent()->getScope();
  const DeviceInfo& devInfo = pMainWnd->getDeviceInfo();
  unsigned int n = pScopeWnd->getNumTrace();
  Glib::RefPtr<Gdk::Window> window = m_drawing.get_window();
  if (window) {
    Gtk::Allocation allocation = get_allocation();
    double w = (double)allocation.get_width();
    double h = (double)allocation.get_height();
    Cairo::RefPtr<Cairo::Context> cr = window->create_cairo_context();
    Cairo::TextExtents ex;
    double x, y;

    if (pEvent) {
      cr->rectangle(pEvent->area.x, pEvent->area.y, pEvent->area.width, pEvent->area.height);
      cr->clip();
    }

    cr->set_source_rgb(0.25, 0.25, 0.25);
    cr->paint();

    cr->set_source_rgb(1.0, 1.0, 1.0);
    cr->set_antialias(Cairo::ANTIALIAS_NONE);
    cr->set_line_width(1.0);
    cr->move_to(0.5, 0.5);
    cr->line_to(w - 0.5, 0.5);
    cr->line_to(w - 0.5, h - 0.5);
    cr->line_to(0.5, h - 0.5);
    cr->line_to(0.5, 0.5);
    cr->stroke();

    cr->set_font_size(14.0);
    cr->get_text_extents(g_pMessage, ex);
    cr->move_to(3.5, 3.5 - ex.y_bearing);
    cr->show_text(g_pMessage);

    x = 20.5;
    y = 35.5;
    for (unsigned int i = 0; i < n; i++) {
      color_t color;
      unsigned int sensorIndex;
      TCHAR* pSensorName;

      sensorIndex = pScopeWnd->getTrace(i);
      ASSERT(devInfo.m_pSensors[sensorIndex].m_bValid);
      pSensorName = devInfo.m_pSensors[sensorIndex].m_info.Description;

      color = devInfo.m_pSensors[sensorIndex].m_traceColor;
      cr->set_source_rgb((double)RED(color) / 255.0, (double)GREEN(color) / 255.0, (double)BLUE(color) / 255.0);
      cr->move_to(x, y);
      cr->line_to(x + 30.0, y);
      cr->stroke();
      cr->get_text_extents(pSensorName, ex);
      cr->move_to(x + 35.0, y - ex.y_bearing * 0.5);
      cr->show_text(pSensorName);
      y += 20.0;
    }
  }

  return false;
}

void
CScopeKey::onRealize()
{
  ISysMonWnd* pMainWnd = m_pParent->getParent()->getParent();
  IScopeWnd* pScopeWnd = m_pParent->getParent()->getScope();
  const DeviceInfo& devInfo = pMainWnd->getDeviceInfo();
  unsigned int n = pScopeWnd->getNumTrace();
  Glib::RefPtr <Gdk::Window> window = get_window();
  if (window) {
    Cairo::RefPtr<Cairo::Context> cr = window->create_cairo_context();
    Cairo::TextExtents ex;
    int maxW = 0;

    cr->set_antialias(Cairo::ANTIALIAS_NONE);
    cr->set_line_width(1.0);
    cr->set_font_size(14.0);

    cr->get_text_extents(g_pMessage, ex);
    if (ex.width > maxW) {
      maxW = 3 + ex.width;
    }

    for (unsigned int i = 0; i < n; i++) {
      unsigned int sensorIndex;
      TCHAR* pSensorName;
      sensorIndex = pScopeWnd->getTrace(i);
      ASSERT(devInfo.m_pSensors[sensorIndex].m_bValid);
      pSensorName = devInfo.m_pSensors[sensorIndex].m_info.Description;
      cr->get_text_extents(pSensorName, ex);
      if (ex.width + 55 > maxW) {
        maxW = ex.width + 55;
      }
    }
    resize(maxW + 20, 45 + 20 * n);
  }
}
