#include "sensorinfownd.h"
#include "sensorinfownd.h"

#define ARRAY_LENGTH(x) (sizeof(x) / sizeof((x)[0]))

static const TCHAR* g_prefixStr[] = {
  // From atto- (10^-18) up to exa- (10^15)
  _T("a"), _T("f"), _T("p"), _T("n"), _T("u"), _T("m"), _T(""), _T("k"), _T("M"), _T("G"), _T("T"), _T("P"), _T("E")
};
static const float g_factor[] = {
  1.0e18f, 1.0e15f, 1.0e12f, 1.0e9f, 1.0e6f, 1.0e3f, 1.0f, 1.0e-3f, 1.0e-6f, 1.0e-9f, 1.0e-12f, 1.0e-15f, 1.0e-18f
};

// -------------------------------------------------------------------------
// Start of ISensorInfoWnd interface
// -------------------------------------------------------------------------

void
CSensorInfoWnd::deviceChanged()
{
  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  if (devInfo.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE) {
    m_listCtl.clear_items();
  }
  if (ADMXRC3_HANDLE_INVALID_VALUE != devInfo.m_hDevice) {
    for (unsigned int i = 0; i < devInfo.m_numSensor; i++) {
      SensorInfo* pSensorInfo;
      std::ostringstream ss;
      const TCHAR* pTmp;
      int exponent, prefix;
      ss.str("");
      ss << i;
      m_listCtl.append_text(ss.str().c_str());
      pSensorInfo = &devInfo.m_pSensors[i];
      if (pSensorInfo->m_bValid) {
        m_listCtl.set_text(i, 1, pSensorInfo->m_info.Description);
        ss.str("");
	if (pSensorInfo->m_info.Unit != ADMXRC3_UNIT_NONE) {
	  exponent = pSensorInfo->m_info.Exponent;
	  prefix = (exponent + 18) / 3;
	  if (prefix < 0 || prefix > (int)ARRAY_LENGTH(g_prefixStr)) {
	    ss << "?";
	  } else {
	    ss << g_prefixStr[prefix];
	  }
	}
        pTmp = CAdmxrc3Help::getUnitString(pSensorInfo->m_info.Unit);
        if (NULL != pTmp) {
          ss << pTmp;
        } else {
          ss << "?";
        }
        m_listCtl.set_text(i, 3, ss.str().c_str());
      } else {
        m_listCtl.set_text(i, 1, "(Unable to determine)");
      }
    }
  }
}

void
CSensorInfoWnd::numDeviceChanged()
{
  unsigned int numDevice = m_pParent->getDeviceInfo().m_numDevice;

  if ((m_numDevice == 0) == (numDevice == 0)) {
    return;
  }

  /* Nothing to do yet */

  m_numDevice = numDevice;
}

void
CSensorInfoWnd::update()
{
  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  SensorInfo* pSensorInfo;
  unsigned int n = devInfo.m_numSensor;

  for (unsigned int i = 0; i < n; i++) {
    std::ostringstream ss;

    pSensorInfo = &devInfo.m_pSensors[i];
    if (pSensorInfo->m_bValid) {
      switch (pSensorInfo->m_info.DataType) {
      case ADMXRC3_DATA_BOOL:
        ss << pSensorInfo->m_current.Bool;
        break;

      case ADMXRC3_DATA_DOUBLE:
        ss.precision(3);
        ss << pSensorInfo->m_current.Double;
        break;

      case ADMXRC3_DATA_INT32:
        ss << pSensorInfo->m_current.Int32;
        break;

      case ADMXRC3_DATA_UINT32:
        ss << pSensorInfo->m_current.UInt32;
        break;

      default:
        ss.str("?");
        break;
      }
    } else {
      ss.str("");
    }
    m_listCtl.set_text(i, 2, ss.str().c_str());
  }
}

// -------------------------------------------------------------------------
// End of ISensorInfoWnd interface
// -------------------------------------------------------------------------

CSensorInfoWnd::CSensorInfoWnd(
  ISysMonWnd* pParent,
  bool_t bVisible) : m_listCtl(4)
{
  m_pParent = pParent;
  m_numDevice = 0;

  set_size_request(-1, 300);

  m_listCtl.set_column_title(0, "#");
  m_listCtl.set_column_title(1, "Description");
  m_listCtl.set_column_title(2, "Value");
  m_listCtl.set_column_title(3, "Unit");

  add(m_listCtl);
  set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
}

CSensorInfoWnd::~CSensorInfoWnd()
{
}
