#include "stdafx.h"
#include "afxwinappex.h"

#ifdef _DEBUG
# define new DEBUG_NEW
#endif

#include <platform.h>
#include <objmodel.h>

#include "control_id.h"
#include "graphwndtoolbar.h"
#include "resource_button.h"
#include "resource_icon.h"

IMPLEMENT_DYNAMIC(CGraphWndToolbar, CWnd)

BEGIN_MESSAGE_MAP(CGraphWndToolbar, CWnd)
  ON_COMMAND(IDCTL_AMP, OnButtonAmp)
  ON_COMMAND(IDCTL_CLOSE, OnButtonClose)
  ON_COMMAND(IDCTL_TEMP, OnButtonTemp)
  ON_COMMAND(IDCTL_BOOL, OnButtonBool)
  ON_COMMAND(IDCTL_GO, OnButtonGo)
  ON_COMMAND(IDCTL_PAUSE, OnButtonPause)
  ON_COMMAND(IDCTL_PLUS, OnButtonPlus)
  ON_COMMAND(IDCTL_VOLT, OnButtonVolt)
  ON_NOTIFY_RANGE(NM_CUSTOMDRAW, 0, 0xFFFFU, OnCustomDraw)
  ON_NOTIFY_RANGE(TTN_SHOW, 0, 0xFFFFU, OnToolTipShow)
  ON_WM_CREATE()
  ON_WM_SIZE()
  ON_WM_CTLCOLOR()
END_MESSAGE_MAP()

// -------------------------------------------------------------------------
// Start of IGraphWndToolbar interface
// -------------------------------------------------------------------------

void
CGraphWndToolbar::deviceChanged()
{
  ISysMonWnd* pMainWnd = m_pParent->getParent();
  const DeviceInfo& devInfo = pMainWnd->getDeviceInfo();
  SensorInfo* pSensorInfo;
  bool_t bEnableTemp = FALSE;
  bool_t bEnableVolt = FALSE;
  bool_t bEnableAmp = FALSE;
  bool_t bEnableBool = FALSE;

  m_goCtl.ShowWindow(FALSE);
  m_pauseCtl.ShowWindow(TRUE);
  m_bPaused = FALSE;

  for (unsigned int i = 0; i < devInfo.m_numSensor; i++) {
    pSensorInfo = &devInfo.m_pSensors[i];
    if (pSensorInfo->m_bValid && pSensorInfo->m_bKnownDataType) {
      switch (pSensorInfo->m_info.Unit) {
      case ADMXRC3_UNIT_C:
        bEnableTemp = TRUE;
        break;

      case ADMXRC3_UNIT_V:
        bEnableVolt = TRUE;
        break;

      case ADMXRC3_UNIT_A:
        bEnableAmp = TRUE;
        break;

      case ADMXRC3_DATA_BOOL:
        bEnableBool = TRUE;
        break;

      default:
        break;
      }
    }
  }
  m_tempCtl.EnableWindow(bEnableTemp);
  m_voltCtl.EnableWindow(bEnableVolt);
  m_ampCtl.EnableWindow(bEnableAmp);
  m_boolCtl.EnableWindow(bEnableBool);
}

void
CGraphWndToolbar::numGraphWndChanged()
{
  ISysMonWnd* pMainWnd = m_pParent->getParent();
  unsigned int n;

  n = pMainWnd->getNumGraphWnd();
  m_plusCtl.EnableWindow(n < 4);
  m_closeCtl.EnableWindow(n >= 2);
}

void
CGraphWndToolbar::numTraceChanged()
{
  unsigned int n = m_pParent->getScope()->getNumTrace();

  if (n > 0) {
    m_keyTipCtl.Activate(TRUE);
    m_goCtl.EnableWindow(TRUE);
    m_pauseCtl.EnableWindow(TRUE);
  } else {
    m_goCtl.EnableWindow(FALSE);
    m_pauseCtl.EnableWindow(FALSE);
    m_keyTipCtl.Activate(FALSE);
  }
  m_requiredKeySize.x = 200;
  m_requiredKeySize.y = 30 + n * 20;
}

void
CGraphWndToolbar::setIndex(
  unsigned int index)
{
  CString caption;
  
  m_index = index;
  caption.Format(_T("%u"), m_index + 1);
  m_captionCtl.SetWindowText(caption);
}

// -------------------------------------------------------------------------
// End of IGraphWndToolbar interface
// -------------------------------------------------------------------------

CGraphWndToolbar::CGraphWndToolbar(
  IGraphWnd* pParent,
  unsigned int index) :
  m_tempCtl(IDI_TEMP),
  m_voltCtl(IDI_VOLT),
  m_ampCtl(IDI_AMP),
  m_boolCtl(IDI_BOOL),
  m_pauseCtl(IDI_PAUSE),
  m_goCtl(IDI_GO),
  m_plusCtl(IDI_PLUS),
  m_closeCtl(IDI_CLOSE)
{
  const TCHAR* pWndClass = NULL;
  bool_t bSuccess;

  m_pParent = pParent;
  m_index = index;
  m_bPaused = FALSE;

  m_bgBrush.CreateSysColorBrush(COLOR_BTNFACE);
  m_fgBrush.CreateSysColorBrush(COLOR_CAPTIONTEXT);
  m_font.CreateStockObject(DEFAULT_GUI_FONT);
  m_keyBgBrush.CreateSolidBrush(RGB(0, 0, 0));

  pWndClass = AfxRegisterWndClass(0, ::LoadCursor(NULL, IDC_ARROW), (HBRUSH)m_bgBrush, NULL);
  bSuccess = CreateEx(WS_EX_CONTROLPARENT, pWndClass, _T(""), WS_CHILD | WS_VISIBLE, 0, 0, 1, 1, *dynamic_cast<CWnd*>(m_pParent), NULL, NULL);
  ASSERT(bSuccess);
}

CGraphWndToolbar::~CGraphWndToolbar()
{
}

void
CGraphWndToolbar::addSensors(
  ADMXRC3_UNIT_TYPE unit)
{
  ISysMonWnd* pMainWnd = m_pParent->getParent();
  IScopeWnd* pScope = m_pParent->getScope();
  const DeviceInfo& devInfo = pMainWnd->getDeviceInfo();
  unsigned int n = devInfo.m_numSensor;

  pScope->removeAllTraces();
  for (unsigned int i = 0; i < n; i++) {
    if (devInfo.m_pSensors[i].m_bKnownDataType && devInfo.m_pSensors[i].m_info.Unit == unit) {
      pScope->addTrace(i);
    }
  }
}

void
CGraphWndToolbar::addSensors(
  ADMXRC3_DATA_TYPE datatype)
{
  ISysMonWnd* pMainWnd = m_pParent->getParent();
  IScopeWnd* pScope = m_pParent->getScope();
  const DeviceInfo& devInfo = pMainWnd->getDeviceInfo();
  unsigned int n = devInfo.m_numSensor;

  pScope->removeAllTraces();
  for (unsigned int i = 0; i < n; i++) {
    if (devInfo.m_pSensors[i].m_bKnownDataType && devInfo.m_pSensors[i].m_info.DataType == datatype) {
      pScope->addTrace(i);
    }
  }
}

void
CGraphWndToolbar::adjustScale(
  float value)
{
  IScopeWnd* pScope = m_pParent->getScope();

  pScope->adjustScale(value);
}

void
CGraphWndToolbar::drawKey(
  HDC hDC)
{
  unsigned int n = m_pParent->getScope()->getNumTrace();
  const DeviceInfo& devInfo = m_pParent->getParent()->getDeviceInfo();
  TCHAR* pSensorName;
  int x, y;
  RECT r1, r2;

  m_keyTipCtl.GetClientRect(&r1);
  SelectObject(hDC, GetStockObject(WHITE_PEN));
  MoveToEx(hDC, r1.left, r1.top, NULL);
  LineTo(hDC, r1.right - 1, r1.top);
  LineTo(hDC, r1.right - 1, r1.bottom - 1);
  LineTo(hDC, r1.left, r1.bottom - 1);
  LineTo(hDC, r1.left, r1.top);

  x = r1.left + 20;
  y = r1.top + 30;
  SelectObject(hDC, GetStockObject(DC_PEN));
  for (unsigned int i = 0; i < n; i++) {
    color_t color;
    unsigned int sensorIndex;

    sensorIndex = m_pParent->getScope()->getTrace(i);
    ASSERT(devInfo.m_pSensors[sensorIndex].m_bValid);
    pSensorName = devInfo.m_pSensors[sensorIndex].m_info.Description;

    color = devInfo.m_pSensors[sensorIndex].m_traceColor;
    SetTextColor(hDC, color);
    SetDCPenColor(hDC, color);

    MoveToEx(hDC, x, y, NULL);
    LineTo(hDC, x + 30, y);
    r2.left = x + 35;
    r2.top = y - 10;
    r2.right = r1.right;
    r2.bottom = y + 10;
    DrawText(hDC, pSensorName, -1, &r2, DT_LEFT | DT_VCENTER | DT_SINGLELINE);
    y += 20;
  }
}

bool_t
CGraphWndToolbar::PreTranslateMessage(
  MSG* pMsg)
{
  m_toolTipCtl.RelayEvent(pMsg);
  m_keyTipCtl.RelayEvent(pMsg);

  return CWnd::PreTranslateMessage(pMsg);
}

void
CGraphWndToolbar::OnButtonAmp()
{
  addSensors(ADMXRC3_UNIT_A);
  adjustScale(0.0f);
  adjustScale(0.01f);
}

void
CGraphWndToolbar::OnButtonBool()
{
  addSensors(ADMXRC3_DATA_BOOL);
  adjustScale(0.0f);
  adjustScale(1.0f);
}

void
CGraphWndToolbar::OnButtonClose(
  void)
{
  m_pParent->getParent()->removeGraphWnd(m_pParent);
}

void
CGraphWndToolbar::OnButtonGo(
  void)
{
  bool_t bSetFocus = FALSE;

  bSetFocus = (GetFocus() == &m_goCtl);
  m_goCtl.ShowWindow(FALSE);
  m_pauseCtl.ShowWindow(TRUE);
  if (bSetFocus) {
    m_pauseCtl.SetFocus();
  }
  m_bPaused = FALSE;
}

void
CGraphWndToolbar::OnButtonPause(
  void)
{
  bool_t bSetFocus = FALSE;

  bSetFocus = (GetFocus() == &m_pauseCtl);
  m_pauseCtl.ShowWindow(FALSE);
  m_goCtl.ShowWindow(TRUE);
  if (bSetFocus) {
    m_goCtl.SetFocus();
  }
  m_bPaused = TRUE;
}

void
CGraphWndToolbar::OnButtonPlus(
  void)
{
  m_pParent->getParent()->addGraphWnd();
}

void
CGraphWndToolbar::OnButtonTemp()
{
  addSensors(ADMXRC3_UNIT_C);
  adjustScale(0.0f);
  adjustScale(100.0f);
}

void
CGraphWndToolbar::OnButtonVolt()
{
  addSensors(ADMXRC3_UNIT_V);
  adjustScale(0.0f);
  adjustScale(1.0f);
}

int
CGraphWndToolbar::OnCreate(
  LPCREATESTRUCT lpCreateStruct)
{
  RECT r = { 0, 0, 0, 0 };
  CString caption;

  if (CWnd::OnCreate(lpCreateStruct) == -1) {
    return -1;
  }

  caption.Format(_T("%u"), m_index + 1);
  m_captionCtl.Create(caption, WS_CHILD | SS_ENDELLIPSIS | SS_CENTERIMAGE, r, this, (UINT)IDCTL_CAPTION);
  m_captionCtl.SetFont(&m_font);
  m_tempCtl.Create(_T(""), WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_FLAT | BS_ICON | BS_CENTER | BS_VCENTER, r, this, (UINT)IDCTL_TEMP);
  m_tempCtl.EnableWindow(FALSE);
  m_voltCtl.Create(_T(""), WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_FLAT | BS_ICON | BS_CENTER | BS_VCENTER, r, this, (UINT)IDCTL_VOLT);
  m_voltCtl.EnableWindow(FALSE);
  m_ampCtl.Create(_T(""), WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_FLAT | BS_ICON | BS_CENTER | BS_VCENTER, r, this, (UINT)IDCTL_AMP);
  m_ampCtl.EnableWindow(FALSE);
  m_boolCtl.Create(_T(""), WS_CHILD | WS_TABSTOP | BS_FLAT | BS_ICON | BS_CENTER | BS_VCENTER, r, this, (UINT)IDCTL_BOOL);
  m_boolCtl.EnableWindow(FALSE);
  m_keyCtl.Create(_T("Key"), WS_CHILD | WS_VISIBLE | SS_CENTER | SS_CENTERIMAGE, r, this, (UINT)IDCTL_KEY);
  m_keyCtl.SetFont(&m_font);
  m_keyCtl.EnableWindow(TRUE);
  m_keyCtl.ModifyStyle(0, SS_NOTIFY, 0); // In order to get tooltips
  m_pauseCtl.Create(_T(""), WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_FLAT | BS_ICON | BS_CENTER | BS_VCENTER, r, this, (UINT)IDCTL_PAUSE);
  m_pauseCtl.EnableWindow(FALSE);
  m_goCtl.Create(_T(""), WS_CHILD | WS_TABSTOP | BS_FLAT | BS_ICON | BS_CENTER | BS_VCENTER, r, this, (UINT)IDCTL_GO);
  m_goCtl.EnableWindow(FALSE);
  m_plusCtl.Create(_T(""), WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_FLAT | BS_ICON | BS_CENTER | BS_VCENTER, r, this, (UINT)IDCTL_PLUS);
  m_plusCtl.EnableWindow(FALSE);
  m_closeCtl.Create(_T(""), WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_FLAT | BS_ICON | BS_CENTER | BS_VCENTER, r, this, (UINT)IDCTL_CLOSE);
  m_closeCtl.EnableWindow(FALSE);

  m_toolTipCtl.CreateEx(this, TTS_ALWAYSTIP);
  m_toolTipCtl.SetFont(&m_font);
  m_toolTipCtl.AddTool(&m_tempCtl, _T("Show all temperature sensors in scope"));
  m_toolTipCtl.AddTool(&m_voltCtl, _T("Show all voltage sensors in scope"));
  m_toolTipCtl.AddTool(&m_ampCtl, _T("Show all current sensors in scope"));
  m_toolTipCtl.AddTool(&m_boolCtl, _T("Show all boolean sensors in scope"));
  m_toolTipCtl.AddTool(&m_pauseCtl, _T("Pause"));
  m_toolTipCtl.AddTool(&m_goCtl, _T("Resume"));
  m_toolTipCtl.AddTool(&m_closeCtl, _T("Close this scope"));
  m_toolTipCtl.AddTool(&m_plusCtl, _T("Add another scope"));
  m_toolTipCtl.Activate(TRUE);

  m_keyTipCtl.CreateEx(this, TTS_ALWAYSTIP);
  m_keyTipCtl.SetFont(&m_font);
  m_keyTipCtl.AddTool(&m_keyCtl, _T("Key to traces:"));
  m_keyTipCtl.SetTipBkColor(RGB(0x40, 0x40, 0x40));
  m_keyTipCtl.SetTipTextColor(RGB(0xff, 0xff, 0xff));
  m_keyTipCtl.SetDelayTime(TTDT_AUTOPOP, 20000);
  m_keyTipCtl.SetDelayTime(TTDT_INITIAL, 50);
  m_keyTipCtl.Activate(TRUE);

  deviceChanged();

  return 0;
}

HBRUSH CGraphWndToolbar::OnCtlColor(
  CDC* pDC,
  CWnd* pWnd,
  UINT /* nCtlColor */)
{
  if (pWnd == &m_captionCtl || pWnd == &m_keyCtl) {
    pDC->SetBkMode(TRANSPARENT);
    pDC->SetTextColor(COLOR_MENUTEXT);
  }
  return (HBRUSH)m_bgBrush;
}

/*
** Order of notifications is:
** PREPAINT
*/
void
CGraphWndToolbar::OnCustomDraw(
  UINT /* nControlId */,
  NMHDR* pNotifyStruct,
  LRESULT* pResult)
{
  NMTTCUSTOMDRAW* pParams = (NMTTCUSTOMDRAW*)pNotifyStruct;

  if (pParams->nmcd.hdr.hwndFrom == m_keyTipCtl.m_hWnd) {
    switch (pParams->nmcd.dwDrawStage) {
    case CDDS_PREPAINT:
      *pResult = CDRF_NOTIFYPOSTPAINT;
      return;

    case CDDS_POSTPAINT:
      drawKey(pParams->nmcd.hdc);
      break;
    }

    *pResult = CDRF_DODEFAULT;
  }
}

void
CGraphWndToolbar::OnSize(
  UINT nType,
  int cx,
  int cy)
{
  const unsigned int buttonSize = 28;
  int x, dy;

  CWnd::OnSize(nType, cx, cy);

  dy = (cy - buttonSize) >> 1;
  x = dy;
  m_tempCtl.MoveWindow(x, dy, buttonSize, buttonSize);
  x += buttonSize + 6;
  m_voltCtl.MoveWindow(x, dy, buttonSize, buttonSize);
  x += buttonSize + 6;
  m_ampCtl.MoveWindow(x, dy, buttonSize, buttonSize);
  x += buttonSize + 6;
  m_keyCtl.MoveWindow(x, dy, 40, buttonSize);
  x += 44;
  m_pauseCtl.MoveWindow(x, dy, buttonSize, buttonSize);
  m_goCtl.MoveWindow(x, dy, buttonSize, buttonSize);
  x += buttonSize + 6;
  dy = ((cy - buttonSize) >> 1);
  m_plusCtl.MoveWindow(cx - (2 * buttonSize + 6 + dy), dy, buttonSize, buttonSize);
  m_closeCtl.MoveWindow(cx - (buttonSize + dy), dy, buttonSize, buttonSize);
}

void
CGraphWndToolbar::OnToolTipShow(
  UINT /* nControlId */,
  NMHDR* pNotifyStruct,
  LRESULT* pResult)
{
  if (pNotifyStruct->hwndFrom == m_keyTipCtl.m_hWnd) {
    RECT r;

    m_keyTipCtl.GetWindowRect(&r);
    m_keyTipCtl.SetWindowPos(NULL, r.left + 20, r.top - 20, m_requiredKeySize.x, m_requiredKeySize.y, SWP_NOZORDER | SWP_NOACTIVATE);
    *pResult = 1;
  } else {
    *pResult = 0; // Default action
  }
}
