#pragma once

#include <objmodel.h>

#ifdef _DEBUG
# define new DEBUG_NEW
#endif

class CSysMonWnd : public CWnd, public ISysMonWnd
{
	DECLARE_DYNAMIC(CSysMonWnd)

public:
	CSysMonWnd();
	virtual ~CSysMonWnd();
  // ISysMonWnd interface
  bool_t addGraphWnd(void);
  const DeviceInfo& getDeviceInfo() const { return m_device; }
  IDeviceInfoWnd* getDeviceInfoWnd() const { return m_pDeviceInfoWnd; }
  IGraphWnd* getGraphWnd(unsigned int index) const {
    ASSERT(index < m_numGraphWnd);
    return m_ppGraphWnd[index];
  }
  window_handle_t getNativeWindow() { return *this; }
  unsigned int getNumGraphWnd() const { return m_numGraphWnd; }
  ISensorInfoWnd* getSensorInfoWnd() const { return m_pSensorInfoWnd; }
  void idle();
  bool_t removeGraphWnd(unsigned int index);

protected:
  // Following members define main window default and minimum size, in pixels
  static const int m_defaultWidth = 640;
  static const int m_defaultHeight = 400;
  static const int m_minWidth = 640;
  static const int m_minHeight = 400;

  // Sensor update period, in milliseconds
  static const int m_sensorUpdatePeriod = 1000;

  // Timer ID used for periodic sensor reads
  static const UINT_PTR m_timerId = 1U;

  unsigned int m_deviceIndex;
  unsigned int m_numGraphWnd;
  int m_currentTab;
  int m_updatePeriodIndex;
  CMenu m_trayMenu;
  CTabCtrl m_tabCtl;
  CFont m_font;
  CBrush m_bgBrush;
  CStatic m_deviceLabelCtl;
  CComboBox m_deviceSelectCtl;
  CStatic m_freqLabelCtl;
  CComboBox m_freqSelectCtl;
  IGraphWnd** m_ppGraphWnd;
  IDeviceInfoWnd* m_pDeviceInfoWnd;
  ISensorInfoWnd* m_pSensorInfoWnd;
  DeviceInfo m_device;
  HICON m_hIcon;
  HICON m_hIconTray;
  bool_t m_bMinimizedToTray;

protected:
  void enumerateDevices(ADMXRC3_CARD_INFOEX* pCardInfoEx);
  virtual bool_t PreTranslateMessage(MSG* pMsg);
  void readSensors();
  bool_t removeGraphWnd(IGraphWnd* pGraphWnd);
  bool_t setDevice(unsigned int deviceIndex, bool_t bFromControl = FALSE);
  bool_t unsetDevice();
  void setNumGraphWnd(unsigned int n);
  bool_t trayIconHelper(DWORD dwMessage, HICON hIcon);
  LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);

  DECLARE_MESSAGE_MAP()

protected:
  afx_msg void OnClose();
  afx_msg void OnClickedFinish();
  afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
  afx_msg BOOL OnDeviceChange(UINT nEventType, DWORD_PTR eventData);
  afx_msg void OnSelectDevice();
  afx_msg void OnSelectUpdatePeriod();
  afx_msg void OnSize(UINT nType, int cx, int cy);
  afx_msg void OnSizing(UINT nSide, RECT* pRect);
  afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
  afx_msg void OnTabChange(NMHDR* pNotifyStruct, LRESULT* pResult);
  afx_msg void OnTimer(UINT_PTR nTimerId); 
  afx_msg void OnTrayExit(); 
  afx_msg void OnTrayRestore(); 
};
