#ifndef _ADATA_PLATFORM_MAIN_H
#define _ADATA_PLATFORM_MAIN_H

#include <vxWorks.h>
#include <taskLib.h>
#include <ctype.h>
#include <string.h>

static int
wrapMain(
  char* pProgramName,
  const char* pArgumentString,
  int (*pMainFunction)(int argc, char* argv[]))
{
  char* pString = NULL;
  char** argv = NULL;
  int i, ret, argc;
  size_t j, n;
  char c, last;

  /* Lower our priority so we don't hog a CPU */
  taskPrioritySet(taskIdSelf(), 255);

  argc = 1;
  if (NULL != pArgumentString) {
    n = strlen(pArgumentString);
    pString = (char*)malloc(sizeof(char) * (n + 1));
    if (NULL == pString) {
      fprintf(stderr, "*** %s: couldn't allocate copy of args string (%u byte(s))\n", pProgramName, sizeof(char) * (n + 1));
      return -1;
    }
    strcpy(pString, pArgumentString);
    last = ' ';
    for (j = 0; j < n; j++) {
      c = pString[j];
      if (!isspace(c) && isspace(last)) {
        argc++;
      }
      last = c;
    }
  }
  argv = (char**)malloc(sizeof(char*) * argc);
  if (NULL == argv) {
    fprintf(stderr, "*** %s: couldn't allocate argv vector (%u byte(s))\n", pProgramName, sizeof(char) * argc);
    free(pString);
    return -1;
  }
  argv[0] = pProgramName;
  i = 1;
  if (NULL != pString) {
    last = ' ';
    for (j = 0; j < n && i < argc; j++) {
      c = pString[j];
      if (!isspace(c)) {
        if (isspace(last)) {
          argv[i] = pString + j;
          i++;
        }
      } else {
        if (!isspace(last)) {
          pString[j] = '\0';
        }
      }
      last = c;
    }
  }
  ret = pMainFunction(argc, argv);
  free(argv);
  free(pString);
  return ret;
}

# define DEFINE_MAIN(prog) static int prog##Main(int argc, TCHAR* argv[])
# define PROGRAM_NAME(prog) int prog(const char* pArgumentString) { return wrapMain(#prog, pArgumentString, prog##Main); }

#endif
