#ifndef _ADATA_PLATFORM_THREAD_H
#define _ADATA_PLATFORM_THREAD_H

#include <vxWorks.h>
#include <sysLib.h>
#include <taskLib.h>
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>

typedef void* (PortableThreadFunction)(void*);

typedef struct _PortableThread {
	int taskId;
	PortableThreadFunction*	pFunction;
	void* pArgument;
  void* pExitCode;
  SEM_ID exitSemId;
} PortableThread;

static void*
_portableThreadStub(
  int arg1,
  int arg2,
  int arg3,
  int arg4,
  int arg5,
  int arg6,
  int arg7,
  int arg8,
  int arg9,
  int arg10)
{
  PortableThread* pThread = (PortableThread*)(uintptr_t)arg1;
  void* pExitCode;

  taskSafe();
  pExitCode = pThread->pFunction(pThread->pArgument);
  pThread->pExitCode = pExitCode;
  taskUnsafe();
  semGive(pThread->exitSemId);

  return pExitCode;
}

#if defined(_WRS_INLINE)
_WRS_INLINE PortableThread*
#else
static __inline__ PortableThread*
#endif
portableThreadCreate(
  PortableThreadFunction* pFunction,
  void* pArgument)
{
  PortableThread* pNew;
  int taskId;
  SEM_ID exitSemId;

  pNew = (PortableThread*)malloc(sizeof(*pNew));
  if (NULL == pNew) {
    return NULL;
  }
  pNew->pFunction = pFunction;
  pNew->pArgument = pArgument;

  exitSemId = semBCreate(0, 0);
  if (NULL == exitSemId) {
    free(pNew);
    return NULL;
  }

  taskId = taskSpawn(NULL, 255, VX_FP_TASK, 0x10000, (FUNCPTR)&_portableThreadStub, (int)(uintptr_t)pNew, 0, 0, 0, 0, 0, 0, 0, 0, 0);
  if (ERROR == taskId) {
    free(pNew);
    return NULL;
  }

  pNew->taskId = taskId;
  pNew->exitSemId = exitSemId;

  return pNew;
}

#if defined(_WRS_INLINE)
_WRS_INLINE BOOL
#else
static __inline__ BOOL
#endif
portableThreadWait(
  PortableThread* pThread,
  unsigned int timeoutMs,
  void** ppResult)
{
  const int k = 1000;
  INT64 ticks;

  ticks = ((INT64)timeoutMs * sysClkRateGet() + k - 1) / k;
  if (OK == semTake(pThread->exitSemId, (int)ticks)) {
    taskDelete(pThread->taskId);
    semDelete(pThread->exitSemId);
    *ppResult = pThread->pExitCode;
    free(pThread);
    return TRUE;
  } else {
    return FALSE;
  }
}

#if defined(_WRS_INLINE)
_WRS_INLINE BOOL
#else
static __inline__ BOOL
#endif
portableThreadWaitInfinite(
  PortableThread* pThread,
  void** ppResult)
{
  if (OK == semTake(pThread->exitSemId, WAIT_FOREVER)) {
    taskDelete(pThread->taskId);
    semDelete(pThread->exitSemId);
    *ppResult = pThread->pExitCode;
    free(pThread);
    return TRUE;
  } else {
    return FALSE;
  }
}

#if defined(_WRS_INLINE)
_WRS_INLINE BOOL
#else
static __inline__ BOOL
#endif
portableThreadKill(
  PortableThread* pThread)
{
  taskDeleteForce(pThread->taskId);
  semDelete(pThread->exitSemId);
  free(pThread);
  return TRUE;
}

#endif
