--==============================================================================
--
--     File Name: adb3_target_pkg.vhd
--       Version: 0.2
--          Date: 09 Feb 2011
--         Model: Synthesisable
--
--  Dependencies: Set simulator resolution to ps for accurate simulation
--
--       Company: Alpha Data
--        Design: Package adb3_target_pkg
--
--   Description: Package of components used in target FPGAs using ADB3.
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2009-2011 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 06 Nov 09: Initial design
--                0.2, TW,  09 Feb 2011: Moved some component declarations into
--                  new adb3_ocp_comp package.
--
--==============================================================================

--
-- Exported routines (parenthesis indicated number of overloads):
--
--   ds_base_conv
--   ds_mask_conv
--   dma_base_conv
--   dma_mask_conv
--   mask_vec_width
--   make_defined_s2m
--   make_defined
--
--
-- Exported components:
--
--   mptl_if_target_wrap
--   mptl_if_target_sim
--   mptl64par_interface_target_6tl_top
--   mptl64par_interface_target_6tl
--   mptl128_interface_target_6t1_top
--   mptl128_interface_target_6t1
--


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_misc.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_inc_pkg.all;

package adb3_target_pkg is

  -- Define adb3_ocp_split_nb functions
  function ds_base_conv(
    constant x : in std_logic_vector)
  return adb3_ocp_addr_s;

  function ds_mask_conv(
    constant x : in std_logic_vector)
  return adb3_ocp_addr_s;

  function dma_base_conv(
    constant x : in std_logic_vector)
  return adb3_ocp_addr_s;

  function dma_mask_conv(
    constant x : in std_logic_vector)
  return adb3_ocp_addr_s;

  function mask_vec_width(
    constant x : in adb3_ocp_addr_s)
  return natural;

  function make_defined_s2m(
    constant x : in adb3_ocp_s2mT)
  return adb3_ocp_s2mT;

  function make_defined(
    constant x : in std_logic_vector)
  return std_logic_vector;


  component mptl_if_target_wrap
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
    -- MPTL Interface
    mptl_t2b                 : out   mptl_pins_t;
    mptl_b2t                 : in    mptl_pins_t;
    mptl_clk                 : in    std_logic_dbl_t;
    -- MPTL Initialisation
    ocp_ready                : in    std_logic;
    mptl_sb_b2t              : in    mptl_sb_b2t_t;
    mptl_sb_t2b              : out   mptl_sb_t2b_t;
    -- MPTL Sideband
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

  component mptl_if_target_sim
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
    direct_masters_m2s       : in    adb3_ocp_m2sT_array(0 to DM_CHANNELS-1);
    direct_masters_s2m       : out   adb3_ocp_s2mT_array(0 to DM_CHANNELS-1);
    dma_abort                : in    std_logic_vector(DMA_CHANNELS-1 downto 0);
    -- MPTL Interface
    mptl_b2t                 : in    mptl_pins_t;
    mptl_t2b                 : out   mptl_pins_t;
    mptl_clk                 : in    std_logic_dbl_t;
    mptl_clk_out             : out   std_logic;
    -- MPTL Initialisation
    ocp_ready                : in    std_logic;
    mptl_sb_b2t              : in    mptl_sb_b2t_t;
    mptl_sb_t2b              : out   mptl_sb_t2b_t;
    -- MPTL Sideband
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

  component mptl64par_interface_target_6tl_top
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
    direct_masters_m2s       : in    adb3_ocp_m2sT_array(0 to DM_CHANNELS-1);
    direct_masters_s2m       : out   adb3_ocp_s2mT_array(0 to DM_CHANNELS-1);
    dma_abort                : in    std_logic_vector(DMA_CHANNELS-1 downto 0);
    -- MPTL Interface
    mptl_b2t                 : in    mptl_pins_t;
    mptl_t2b                 : out   mptl_pins_t;
    mptl_clk                 : in    std_logic_dbl_t;
    mptl_clk_out             : out   std_logic;
    -- MPTL Initialisation
    ocp_ready                : in    std_logic;
    mptl_sb_b2t              : in    mptl_sb_b2t_t;
    mptl_sb_t2b              : out   mptl_sb_t2b_t;
    -- MPTL Sideband
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

  component mptl128_interface_target_6t1_top
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
    direct_masters_m2s       : in    adb3_ocp_m2sT_array(0 to DM_CHANNELS-1);
    direct_masters_s2m       : out   adb3_ocp_s2mT_array(0 to DM_CHANNELS-1);
    dma_abort                : in    std_logic_vector(DMA_CHANNELS-1 downto 0);
    -- MPTL Interface
    mptl_b2t                 : in    mptl_pins_t;
    mptl_t2b                 : out   mptl_pins_t;
    mptl_clk                 : in    std_logic_dbl_t;
    mptl_clk_out             : out   std_logic;
    -- MPTL Initialisation
    ocp_ready                : in    std_logic;
    mptl_sb_b2t              : in    mptl_sb_b2t_t;
    mptl_sb_t2b              : out   mptl_sb_t2b_t;
    -- MPTL Sideband
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

  component mptl64par_interface_target_6tl
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to 1);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to 1);
    -- MPTL Interface
    mptl_tx                  : out   mptl_pins_t;
    mptl_rx                  : in    mptl_pins_t;
    mptl_clk                 : in    std_logic_dbl_t;
    mptl_clk_out             : out   std_logic;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_n : out   std_logic;
    mptl_target_gtp_online_n : out   std_logic;
    mptl_bridge_gtp_online_n : in    std_logic;
    -- MPTL Sideband Interface
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

  component mptl128_interface_target_6t1
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to 3);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to 3);
    dma_abort                : in    std_logic_vector(3 downto 0);
    -- MPTL Interface
    mptl_tx                  : out   mptl_pins_t;
    mptl_rx                  : in    mptl_pins_t;
    mptl_clk                 : in    std_logic_dbl_t;
    mptl_clk_out             : out   std_logic;
    -- Bridge to Target Sideband Signals
    mptl_target_configured_n : out   std_logic;
    mptl_target_gtp_online_n : out   std_logic;
    mptl_bridge_gtp_online_n : in    std_logic;
    -- MPTL64 Sideband Interface
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

end;


package body adb3_target_pkg is

  -- Define adb3_ocp_split_nb functions
  -- Convert DS base address to adb3_ocp_addr_s
  function ds_base_conv(
    constant x : in std_logic_vector)
  return adb3_ocp_addr_s is
    variable n : natural := x'length;                    -- Size of x in bits (min size of 1)
    variable m : std_logic_vector(n - 1 downto 0) := x;  -- New vector of size n bits
  begin
    if (n > DS_ADDR_WIDTH) then
      assert not((OR_reduce(m(n-1 downto DS_ADDR_WIDTH))) = '1')
        report "(ds_base_conv): Address larger than DS_ADDR_WIDTH = " & integer'image(DS_ADDR_WIDTH)
        severity failure;
    end if;

    return std_logic_vector(RESIZE(unsigned(x),ADB3_OCP_ADDR_WIDTH));
  end;

  -- Convert DS mask address to adb3_ocp_addr_s
  function ds_mask_conv(
    constant x : in std_logic_vector)
  return adb3_ocp_addr_s is
    variable n : natural := x'length;                    -- Size of x in bits (min size of 1)
    variable m : std_logic_vector(n - 1 downto 0) := x;  -- New vector of size n bits
    variable p : std_logic_vector(ADB3_OCP_ADDR_WIDTH-1 downto DS_ADDR_WIDTH) := (others => '1');
    variable q : adb3_ocp_addr_s;
  begin
    if (n > DS_ADDR_WIDTH) then
      assert not((OR_reduce(m(n-1 downto DS_ADDR_WIDTH))) = '1')
        report "(ds_mask_conv): Address larger than DS_ADDR_WIDTH = " & integer'image(DS_ADDR_WIDTH)
        severity failure;
    end if;

    q := p & std_logic_vector(RESIZE(unsigned(x),DS_ADDR_WIDTH));
    return q;
  end;

  -- Convert DMA base address to adb3_ocp_addr_s
  function dma_base_conv(
    constant x : in std_logic_vector)
  return adb3_ocp_addr_s is
    variable n : natural := x'length;                    -- Size of x in bits (min size of 1)
    variable m : std_logic_vector(n - 1 downto 0) := x;  -- New vector of size n bits
  begin
    if (n > DMA_ADDR_WIDTH) then
      assert not((OR_reduce(m(n-1 downto DMA_ADDR_WIDTH))) = '1')
        report "(dma_base_conv): Address larger than DMA_ADDR_WIDTH = " & integer'image(DMA_ADDR_WIDTH)
        severity failure;
    end if;

    return std_logic_vector(RESIZE(unsigned(x),ADB3_OCP_ADDR_WIDTH));
  end;

  -- Convert DMA mask address to adb3_ocp_addr_s
  function dma_mask_conv(
    constant x : in std_logic_vector)
  return adb3_ocp_addr_s is
    variable n : natural := x'length;                    -- Size of x in bits (min size of 1)
    variable m : std_logic_vector(n - 1 downto 0) := x;  -- New vector of size n bits
    variable p : std_logic_vector(ADB3_OCP_ADDR_WIDTH-1 downto DMA_ADDR_WIDTH) := (others => '1');
    variable q : adb3_ocp_addr_s;
  begin
    if (n > DMA_ADDR_WIDTH) then
      assert not((OR_reduce(m(n-1 downto DMA_ADDR_WIDTH))) = '1')
        report "(dma_mask_conv): Address larger than DMA_ADDR_WIDTH = " & integer'image(DMA_ADDR_WIDTH)
        severity failure;
    end if;

    q := p & std_logic_vector(RESIZE(unsigned(x),DMA_ADDR_WIDTH));
    return q;
  end;

  -- Convert address mask width to natural
  function mask_vec_width(
    constant x : in adb3_ocp_addr_s)
  return natural is
    variable n : natural;
  begin
    n := 0;
    for j in ADB3_OCP_ADDR_WIDTH-1 downto 0 loop
      if (x(j) = '0') then
        n := j;
      end if;
    end loop;

    return n;
  end;

  -- Make sure OCP s2m record has defined data for MPTL serial interface simulation
  function make_defined_s2m(
    constant x : in adb3_ocp_s2mT)
  return adb3_ocp_s2mT is
    variable m : adb3_ocp_s2mT;  -- New OCP record
  begin
    m.CmdAccept  := x.CmdAccept;
    m.DataAccept := x.DataAccept;
    m.Resp       := x.Resp;
    m.Data       := make_defined(x.Data);
    m.Tag        := make_defined(x.Tag);
    return m;
  end;

  -- Convert any non '0' or '1' bits of std_logic_vector to '0'
  function make_defined(
    constant x : in std_logic_vector)
  return std_logic_vector is
    variable n : natural := x'length;               -- Size of x in bits (min size of 1)
    variable m : std_logic_vector(n - 1 downto 0);  -- New vector of size n bits
  begin
    for i in 0 to (n - 1) loop
      if (x(i) = '0')or(x(i) = '1') then
        m(i) := x(i);
      else
        m(i) := '0';
      end if;
    end loop;

    return m;
  end;

end;
