--==============================================================================
--
--     File Name: mptl_if_bridge_wrap.vhd
--       Version: 0.1
--          Date: 23 Mar 2010
--         Model: sim (Not Synthesisable)
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: Bridge MPTL interface wrapper
--
--   Description: Selects between sim and synth versions of bridge MPTL interface
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2010 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 23 Mar 2010: Initial design
--
--==============================================================================


library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_types_pkg.all;
use work.adb3_target_inc_pkg.all;
use work.adb3_target_tb_pkg.all;

entity mptl_if_bridge_wrap is
port(
  -- OCP Interface
  ocp_clk_in               : in    std_logic;
  direct_slave_m2s         : in    adb3_ocp_m2sT;
  direct_slave_s2m         : out   adb3_ocp_s2mT;
  dma_channels_m2s         : in    adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
  dma_channels_s2m         : out   adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
  direct_masters_m2s       : out   adb3_ocp_m2sT_array(0 to DM_CHANNELS-1);
  direct_masters_s2m       : in    adb3_ocp_s2mT_array(0 to DM_CHANNELS-1);
  dma_abort                : out   std_logic_vector(DMA_CHANNELS-1 downto 0);
  ocp_clk_out              : out   std_logic;
  -- MPTL Interface
  mptl_t2b                 : in    mptl_pins_t;
  mptl_b2t                 : out   mptl_pins_t;
  mptl_clk                 : in    std_logic_dbl_t;
  -- MPTL Initialisation
  mptl_sb_t2b              : in    mptl_sb_t2b_t;
  mptl_sb_b2t              : out   mptl_sb_b2t_t;
  -- MPTL Sideband
  gpio_b2t                 : in    std_logic_vector(15 downto 0);
  gpio_t2b                 : out   std_logic_vector(15 downto 0);
  mptl_status              : out   std_logic_vector(63 downto 0);
  mptl_config              : in    std_logic_vector(31 downto 0));
end;


architecture sim of mptl_if_bridge_wrap is

begin

  -- Check BOARD_TYPE values included here
  assert not((BOARD_TYPE /= ADM_XRC_6TL)and(BOARD_TYPE /= ADM_XRC_6T1))
    report "(mptl_if_bridge_wrap): Illegal value: BOARD_TYPE = " & board_type_t'image(BOARD_TYPE)
    severity failure;

  -- Check TARGET_USE values included here
  assert not((TARGET_USE /= SIM_OCP)and(TARGET_USE /= SIM_MPTL)and(TARGET_USE /= SYN_NGC))
    report "(mptl_if_bridge_wrap): Illegal value: TARGET_USE = " & target_use_t'image(TARGET_USE)
    severity failure;


  inst_sim_ocp_g : if (TARGET_USE = SIM_OCP) generate
    -- direct OCP-OCP simulation interface
    mptl_if_bridge_sim_i : mptl_if_bridge_sim
    port map(
      -- OCP Interface
      ocp_clk_in         => ocp_clk_in,          -- in    std_logic
      direct_slave_m2s   => direct_slave_m2s,    -- in    adb3_ocp_m2sT
      direct_slave_s2m   => direct_slave_s2m,    -- out   adb3_ocp_s2mT
      dma_channels_m2s   => dma_channels_m2s,    -- in    adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1)
      dma_channels_s2m   => dma_channels_s2m,    -- out   adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1)
      direct_masters_m2s => direct_masters_m2s,  -- out   adb3_ocp_m2sT_array(0 to DM_CHANNELS-1)
      direct_masters_s2m => direct_masters_s2m,  -- in    adb3_ocp_s2mT_array(0 to DM_CHANNELS-1)
      dma_abort          => dma_abort,           -- out   std_logic_vector(DMA_CHANNELS-1 downto 0);
      ocp_clk_out        => ocp_clk_out,         -- out   std_logic
      -- MPTL Interface
      mptl_t2b           => mptl_t2b,            -- in    mptl_pins_t;
      mptl_b2t           => mptl_b2t,            -- out   mptl_pins_t;
      mptl_clk           => mptl_clk,            -- in    std_logic_dbl_t;
      -- MPTL Initialisation
      mptl_sb_t2b        => mptl_sb_t2b,         -- in    mptl_sb_t2b_t;
      mptl_sb_b2t        => mptl_sb_b2t,         -- out   mptl_sb_b2t_t;
      -- MPTL Sideband
      gpio_b2t           => gpio_b2t,            -- in    std_logic_vector(15 downto 0)
      gpio_t2b           => gpio_t2b,            -- out   std_logic_vector(15 downto 0)
      mptl_status        => mptl_status,         -- out   std_logic_vector(63 downto 0))
      mptl_config        => mptl_config);        -- in    std_logic_vector(31 downto 0))
  end generate;

  inst_sim_mptl_6t1_g : if (TARGET_USE /= SIM_OCP)and(BOARD_TYPE = ADM_XRC_6T1) generate
    -- full OCP-MPTL-OCP simulation interface
    mptl128_interface_bridge_6t1_top_i : mptl128_interface_bridge_6t1_top
    port map(
      -- OCP Interface
      ocp_clk_in         => ocp_clk_in,          -- in    std_logic
      direct_slave_m2s   => direct_slave_m2s,    -- in    adb3_ocp_m2sT
      direct_slave_s2m   => direct_slave_s2m,    -- out   adb3_ocp_s2mT
      dma_channels_m2s   => dma_channels_m2s,    -- in    adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1)
      dma_channels_s2m   => dma_channels_s2m,    -- out   adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1)
      direct_masters_m2s => direct_masters_m2s,  -- out   adb3_ocp_m2sT_array(0 to DM_CHANNELS-1);
      direct_masters_s2m => direct_masters_s2m,  -- in    adb3_ocp_s2mT_array(0 to DM_CHANNELS-1);
      dma_abort          => dma_abort,           -- out   std_logic_vector(DMA_CHANNELS-1 downto 0);
      ocp_clk_out        => ocp_clk_out,         -- out   std_logic
      -- MPTL Interface
      mptl_t2b           => mptl_t2b,            -- in    mptl_pins_t;
      mptl_b2t           => mptl_b2t,            -- out   mptl_pins_t;
      mptl_clk           => mptl_clk,            -- in    std_logic_dbl_t;
      -- MPTL Initialisation
      mptl_sb_t2b        => mptl_sb_t2b,         -- in    mptl_sb_t2b_t;
      mptl_sb_b2t        => mptl_sb_b2t,         -- out   mptl_sb_b2t_t;
      -- MPTL Sideband
      gpio_b2t           => gpio_b2t,            -- in    std_logic_vector(15 downto 0)
      gpio_t2b           => gpio_t2b,            -- out   std_logic_vector(15 downto 0)
      mptl_status        => mptl_status,         -- out   std_logic_vector(63 downto 0)
      mptl_config        => mptl_config);        -- in    std_logic_vector(31 downto 0)
  end generate;


  inst_sim_mptl_6tl_g : if (TARGET_USE /= SIM_OCP)and(BOARD_TYPE = ADM_XRC_6TL) generate
    -- full OCP-MPTL-OCP simulation interface
    mptl64par_interface_bridge_6tl_top_i : mptl64par_interface_bridge_6tl_top
    port map(
      -- OCP Interface
      ocp_clk_in         => ocp_clk_in,          -- in    std_logic
      direct_slave_m2s   => direct_slave_m2s,    -- in    adb3_ocp_m2sT
      direct_slave_s2m   => direct_slave_s2m,    -- out   adb3_ocp_s2mT
      dma_channels_m2s   => dma_channels_m2s,    -- in    adb3_ocp_m2sT_array(0 to 1)
      dma_channels_s2m   => dma_channels_s2m,    -- out   adb3_ocp_s2mT_array(0 to 1)
      direct_masters_m2s => direct_masters_m2s,  -- out   adb3_ocp_m2sT
      direct_masters_s2m => direct_masters_s2m,  -- in    adb3_ocp_s2mT
      dma_abort          => dma_abort,           -- out   std_logic_vector(DMA_CHANNELS-1 downto 0);
      ocp_clk_out        => ocp_clk_out,         -- out   std_logic
      -- MPTL Interface
      mptl_t2b           => mptl_t2b,            -- in    mptl_pins_t;
      mptl_b2t           => mptl_b2t,            -- out   mptl_pins_t;
      mptl_clk           => mptl_clk,            -- in    std_logic_dbl_t;
      -- MPTL Initialisation
      mptl_sb_t2b        => mptl_sb_t2b,         -- in    mptl_sb_t2b_t;
      mptl_sb_b2t        => mptl_sb_b2t,         -- out   mptl_sb_b2t_t;
      -- MPTL Sideband
      gpio_b2t           => gpio_b2t,            -- in    std_logic_vector(15 downto 0)
      gpio_t2b           => gpio_t2b,            -- out   std_logic_vector(15 downto 0)
      mptl_status        => mptl_status,         -- out   std_logic_vector(63 downto 0)
      mptl_config        => mptl_config);        -- in    std_logic_vector(31 downto 0)
  end generate;

end;